@@ -28,10 +28,8 @@
 #include <openssl/x509v3.h>
 #ifdef WIN32
 #include <openssl/rand.h>		  /* RAND_seed() */
-#include "../../config-win32.h"	  /* HAVE_SNPRINTF */
-#else
-#include "../../config.h"
 #endif
+#include "config.h"
 #include <time.h>				  /* asctime() */
 #include <string.h>				  /* strncpy() */
 #include "ssl.h"				  /* struct cert_info */
@@ -46,6 +44,10 @@
 #define SSL_OP_SINGLE_ECDH_USE 0
 #endif
 
+#ifndef SSL_OP_NO_COMPRESSION
+#define SSL_OP_NO_COMPRESSION 0
+#endif
+
 /* globals */
 static struct chiper_info chiper_info;		/* static buffer for _SSL_get_cipher_info() */
 static char err_buf[256];			/* generic error buffer */
@@ -78,16 +80,13 @@ __SSL_critical_error (char *funcname)
 /* +++++ SSL functions +++++ */
 
 SSL_CTX *
-_SSL_context_init (void (*info_cb_func), int server)
+_SSL_context_init (void (*info_cb_func))
 {
 	SSL_CTX *ctx;
-#ifdef WIN32
-	int i, r;
-#endif
 
 	SSLeay_add_ssl_algorithms ();
 	SSL_load_error_strings ();
-	ctx = SSL_CTX_new (server ? SSLv23_server_method() : SSLv23_client_method ());
+	ctx = SSL_CTX_new (SSLv23_client_method ());
 
 	SSL_CTX_set_session_cache_mode (ctx, SSL_SESS_CACHE_BOTH);
 	SSL_CTX_set_timeout (ctx, 300);
@@ -97,18 +96,13 @@ _SSL_context_init (void (*info_cb_func), int server)
 							  |SSL_OP_NO_TICKET
 							  |SSL_OP_CIPHER_SERVER_PREFERENCE);
 
+#if OPENSSL_VERSION_NUMBER >= 0x00908000L && !defined (OPENSSL_NO_COMP) /* workaround for OpenSSL 0.9.8 */
+	sk_SSL_COMP_zero(SSL_COMP_get_compression_methods());
+#endif
+
 	/* used in SSL_connect(), SSL_accept() */
 	SSL_CTX_set_info_callback (ctx, info_cb_func);
 
-#ifdef WIN32
-	/* under win32, OpenSSL needs to be seeded with some randomness */
-	for (i = 0; i < 128; i++)
-	{
-		r = rand ();
-		RAND_seed ((unsigned char *)&r, sizeof (r));
-	}
-#endif
-
 	return(ctx);
 }
 
@@ -477,7 +471,7 @@ _SSL_check_subject_altname (X509 *cert, const char *host)
 
 	if (addr != NULL)
 		g_object_unref (addr);
-	sk_GENERAL_NAME_free (altname_stack);
+	sk_GENERAL_NAME_pop_free (altname_stack, GENERAL_NAME_free);
 	return rv;
 }
 
@@ -498,9 +492,7 @@ _SSL_check_common_name (X509 *cert, const char *host)
 	if (common_name_len < 0)
 		return -1;
 
-	common_name = calloc (common_name_len + 1, 1);
-	if (common_name == NULL)
-		return -1;
+	common_name = g_malloc0 (common_name_len + 1);
 
 	X509_NAME_get_text_by_NID (name, NID_commonName, common_name, common_name_len + 1);
 
@@ -529,7 +521,7 @@ _SSL_check_common_name (X509 *cert, const char *host)
 		rv = 0;
 
 out:
-	free(common_name);
+	g_free(common_name);
 	return rv;
 }
 
@@ -543,4 +535,4 @@ _SSL_check_hostname (X509 *cert, const char *host)
 		return rv;
 
 	return _SSL_check_common_name (cert, host);
-}
\ No newline at end of file
+}
